@@ -1,4 +1,4 @@
-// Copyright (C) 1999 Cygnus Solutions
+// Copyright (C) 1999, 2000 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -26,20 +26,33 @@
 // the GNU General Public License.
 
 //
-// Written by Gabriel Dos Reis <gdr@egcs.cygnus.com>
+// Written by Gabriel Dos Reis <gdr@gcc.gnu.org>
 // 
 // Note: This program outputs speciliazations of ISO C++ class template
 // numeric_limits<> as described in 18.2.1.
 // Do not compile with optimization turned on.
 //
 
+#include <bits/c++config.h>
+
+//
+// Force Linux <limits.h> to define the *LONG_LONG*
+//
+#if __linux__ && _GLIBCPP_USE_LONG_LONG
+# ifndef __USE_GNU
+#  define __USE_GNU 1
+# endif
+# ifndef _GNU_SOURCE
+#  define _GNU_SOURCE 1
+# endif
+#endif
+
 #include <limits.h>
 #include <float.h>
 #include <stdio.h>
 #include <signal.h>
 #include <setjmp.h>
 #include <math.h>
-#include <bits/c++config.h>
 #ifdef _GLIBCPP_USE_WCHAR_T
 #include <wchar.h>
 #endif
@@ -63,7 +76,8 @@ const int integer_base_rep = 2;
 
 //
 // Integer types: bool, char, signed char, unsigned char, wchar_t,
-// short, unsigned short, int, unsigned, long, unsigned long
+// short, unsigned short, int, unsigned, long, unsigned long,
+// and possibly long long and unsigned long long
 //
 // Here ISO 14882 disagrees with LIA-1 in stating bool to be an
 // integer type.  Therefore itn't suprising to see ambiguity in the
@@ -78,7 +92,16 @@ const int integer_base_rep = 2;
 
 jmp_buf env;
 
-void signal_handler(int sig) { longjmp(env, sig); }
+void signal_handler(int sig) 
+{ 
+#ifdef __CYGWIN__
+  static sigset_t x;
+  signal (sig, signal_handler);
+  sigemptyset (&x);
+  sigprocmask(SIG_SETMASK, &x, NULL);
+#endif /* __CYGWIN__ */
+  longjmp(env, sig); 
+}
 
 template<typename Operation>
 bool trapping(const Operation& op)
@@ -134,6 +157,9 @@ SPECIALIZE_TRAPPING(unsigned char);
 SPECIALIZE_TRAPPING(unsigned short);
 SPECIALIZE_TRAPPING(unsigned int);
 SPECIALIZE_TRAPPING(unsigned long);
+#if _GLIBCPP_USE_LONG_LONG
+SPECIALIZE_TRAPPING(unsigned long long);
+#endif
 
 #undef SPECIALIZE_TRAPPING
 
@@ -157,6 +183,10 @@ DEFINED_TYPE_NAME(int);
 DEFINED_TYPE_NAME(unsigned int);
 DEFINED_TYPE_NAME(long);
 DEFINED_TYPE_NAME(unsigned long);
+#ifdef _GLIBCPP_USE_LONG_LONG
+DEFINED_TYPE_NAME(long long);
+DEFINED_TYPE_NAME(unsigned long long);
+#endif
 DEFINED_TYPE_NAME(float);
 DEFINED_TYPE_NAME(double);
 DEFINED_TYPE_NAME(long double);
@@ -250,6 +280,10 @@ SPECIALIZE_IEC559(int);
 SPECIALIZE_IEC559(unsigned int);
 SPECIALIZE_IEC559(long);
 SPECIALIZE_IEC559(unsigned long);
+#ifdef _GLIBCPP_USE_LONG_LONG
+SPECIALIZE_IEC559(long long);
+SPECIALIZE_IEC559(unsigned long long);
+#endif
 
 #undef SPECIALIZE_IEC559
 
@@ -292,6 +326,10 @@ DEFINE_EXTREMA(int, INT_MIN, INT_MAX);
 DEFINE_EXTREMA(unsigned int, 0, UINT_MAX);
 DEFINE_EXTREMA(long, LONG_MIN, LONG_MAX);
 DEFINE_EXTREMA(unsigned long, 0, ULONG_MAX);
+#ifdef _GLIBCPP_USE_LONG_LONG
+DEFINE_EXTREMA(long long, LONG_LONG_MIN, LONG_LONG_MAX);
+DEFINE_EXTREMA(unsigned long long, 0, ULONG_LONG_MAX);
+#endif
 DEFINE_EXTREMA(float, FLT_MIN, FLT_MAX);
 DEFINE_EXTREMA(double, DBL_MIN, DBL_MAX);
 DEFINE_EXTREMA(long double, LDBL_MIN, LDBL_MAX);
@@ -304,7 +342,7 @@ template<typename T>
 const int value<T>::digits =
       bits_per_byte * sizeof(T) - int(predicate<T>::is_signed);
 
-// Non integer types should specialize this.  Alwyas two for
+// Non integer types should specialize this.  Always two for
 // integer types.
 template<typename T>
 const int value<T>::radix = 2;
@@ -328,16 +366,38 @@ SPECIALIZE_RADIX(long double, FLT_RADIX);
 
 #undef SPECIALIZE_RADIX
 
+// Non integer types should specialize this.  
+// Unfortunately, systems that don't deal with weak linking correctly
+// (Ie, hpux and aix), cannot use this sophisticated yet sane method. So,
+// explicitly instantiate all the data members here so that they will
+// be happy.
 
+// sophisticated, sane method
+#if 0
 template<typename T>
 const char value<T>::epsilon[] = "0";
-
-template<typename T>
-const char value<T>::round_error[] = "0";
+#endif
 
 #define SPECIALIZE_EPSILON(T, E) DO_SPECIALIZE_EPSILON(T, E)
 #define DO_SPECIALIZE_EPSILON(T, E) const char value< T >::epsilon[] = #E
 
+// unsophisticated, gross method
+#if 1
+SPECIALIZE_EPSILON(bool, 0);
+SPECIALIZE_EPSILON(char, 0);
+SPECIALIZE_EPSILON(unsigned char, 0);
+SPECIALIZE_EPSILON(signed char, 0);
+SPECIALIZE_EPSILON(wchar_t, 0);
+SPECIALIZE_EPSILON(short, 0);
+SPECIALIZE_EPSILON(unsigned short, 0);
+SPECIALIZE_EPSILON(int, 0);
+SPECIALIZE_EPSILON(unsigned int, 0);
+SPECIALIZE_EPSILON(long, 0);
+SPECIALIZE_EPSILON(unsigned long, 0);
+SPECIALIZE_EPSILON(long long, 0);
+SPECIALIZE_EPSILON(unsigned long long, 0);
+#endif
+
 SPECIALIZE_EPSILON(float, FLT_EPSILON);
 SPECIALIZE_EPSILON(double, DBL_EPSILON);
 SPECIALIZE_EPSILON(long double, LDBL_EPSILON);
@@ -346,7 +406,35 @@ SPECIALIZE_EPSILON(long double, LDBL_EPSILON);
 #undef SPECIALIZE_EPSILON
 
 
+// Non integer types should specialize this.  
+// Unfortunately, systems that don't deal with weak linking correctly
+// (Ie, hpux and aix), cannot use this sophisticated yet sane method. So,
+// explicitly instantiate all the data members here so that they will
+// be happy.
+
+// sophisticated, sane method
+#if 0
+template<typename T>
+const char value<T>::round_error[] = "0";
+#endif
+
 #define SPECIALIZE_ROUND_ERROR(T, R) const char value< T >::round_error[] = #R
+// unsophisticated, gross method
+#if 1
+SPECIALIZE_ROUND_ERROR(bool, 0);
+SPECIALIZE_ROUND_ERROR(char, 0);
+SPECIALIZE_ROUND_ERROR(unsigned char, 0);
+SPECIALIZE_ROUND_ERROR(signed char, 0);
+SPECIALIZE_ROUND_ERROR(wchar_t, 0);
+SPECIALIZE_ROUND_ERROR(short, 0);
+SPECIALIZE_ROUND_ERROR(unsigned short, 0);
+SPECIALIZE_ROUND_ERROR(int, 0);
+SPECIALIZE_ROUND_ERROR(unsigned int, 0);
+SPECIALIZE_ROUND_ERROR(long, 0);
+SPECIALIZE_ROUND_ERROR(unsigned long, 0);
+SPECIALIZE_ROUND_ERROR(long long, 0);
+SPECIALIZE_ROUND_ERROR(unsigned long long, 0);
+#endif
 
 SPECIALIZE_ROUND_ERROR(float, 1.0f);
 SPECIALIZE_ROUND_ERROR(double, 1.0);
@@ -638,7 +726,7 @@ int main()
     type_trait<char>();
     type_trait<signed char>();
     type_trait<unsigned char>();
-#if defined( _GLIBCPP_USE_WCHAR_T) && defined(_GLIBCPP_HAS_WCHAR_MIN_MAX)
+#if defined( _GLIBCPP_USE_WCHAR_T) 
     type_trait<wchar_t>();
 #endif
     
@@ -651,6 +739,11 @@ int main()
     type_trait<long>();
     type_trait<unsigned long>();
 
+#ifdef _GLIBCPP_USE_LONG_LONG
+    type_trait<long long>();
+    type_trait<unsigned long long>();
+#endif
+
     type_trait<float>();
     type_trait<double>();
     type_trait<long double>();
