@@ -1,6 +1,6 @@
 // Locale support -*- C++ -*-
 
-// Copyright (C) 1999 Cygnus Solutions
+// Copyright (C) 1999, 2000 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -35,7 +35,7 @@
 #include <bits/std_cstring.h>
 #include <bits/std_cassert.h>
 #include <bits/std_limits.h>
-#include <bits/std_exception.h>
+#include <exception>
 #include <bits/std_stdexcept.h>
 #include <bits/std_locale.h>
 #include <bits/locale_facets.tcc>
@@ -95,15 +95,16 @@ namespace std {
 #endif
   
   // time_get and time_put
-  //template class time_get<char, obuf_iterator>;
   template class time_put<char, obuf_iterator>;
+  template class time_put_byname<char, obuf_iterator>;
   template class time_get<char, ibuf_iterator>;
-  template class time_put<char, ibuf_iterator>;
+  template class time_get_byname<char, ibuf_iterator>;
+
 #ifdef _GLIBCPP_USE_WCHAR_T
-  //template class time_get<wchar_t, wobuf_iterator>;
   template class time_put<wchar_t, wobuf_iterator>;
+  template class time_put_byname<wchar_t, wobuf_iterator>;
   template class time_get<wchar_t, wibuf_iterator>;
-  template class time_put<wchar_t, wibuf_iterator>;
+  template class time_get_byname<wchar_t, wibuf_iterator>;
 #endif
 
   // messages
@@ -117,17 +118,21 @@ namespace std {
 #endif
   
   // ctype
+  //  template class ctype<unsigned char>; // No definitions avail.
+  //  template class ctype<signed char>; // No definitions avail.
   template class _Ctype<char>;
   template class _Ctype_nois<char>;
   template class ctype_byname<char>;
-  template class _Codecvt<char, char, mbstate_t>;
 #ifdef _GLIBCPP_USE_WCHAR_T
   template class _Ctype<wchar_t>;
   template class _Ctype_nois<wchar_t>;
   template class ctype_byname<wchar_t>;
-  template class _Codecvt<wchar_t, char, mbstate_t>;
 #endif
   
+  // codecvt
+  template class __codecvt_abstract_base<char, char, mbstate_t>;
+  template class __codecvt_abstract_base<wchar_t, char, mbstate_t>;
+
   // collate
   template class _Collate<char>;
   template class collate_byname<char>;
@@ -147,13 +152,10 @@ namespace std {
   template 
     const num_get<char, ibuf_iterator >& 
     use_facet<num_get<char, ibuf_iterator> >(const locale &);
-  template
-    const ctype<char>&
-    use_facet<ctype<char> >(const locale& __loc);
   template
     const codecvt<char, char, mbstate_t>& 
-    use_facet<codecvt<char, char, mbstate_t> >(locale const &);
-  template 
+    use_facet<codecvt<char, char, mbstate_t> >(const locale&);
+   template 
     const num_put<char, obuf_iterator>& 
     _Use_facet_failure_handler<num_put<char, obuf_iterator> >
     (const locale &);
@@ -164,12 +166,6 @@ namespace std {
   template 
     const num_get<wchar_t, wibuf_iterator>& 
     use_facet<num_get<wchar_t, wibuf_iterator> >(const locale &);
-  template
-    const ctype<wchar_t>&
-    use_facet<ctype<wchar_t> >(const locale& __loc);
-  template
-    const codecvt<wchar_t, wchar_t, mbstate_t>& 
-    use_facet<codecvt<wchar_t, wchar_t, mbstate_t> >(locale const &);
   template
     const codecvt<wchar_t, char, mbstate_t>& 
     use_facet<codecvt<wchar_t, char, mbstate_t> >(locale const &);
@@ -179,8 +175,18 @@ namespace std {
     (const locale &);
 #endif
 
+  // has_facet
+  template 
+    bool
+    has_facet<numpunct<char> >(const locale &);
+#ifdef _GLIBCPP_USE_WCHAR_T
+  template 
+    bool
+    has_facet<numpunct<wchar_t> >(const locale &);
+#endif
+
   //
-  // iterator/locale
+  // iterator
   //
   typedef vector<locale::facet*> vec_pfacet;
   template 
@@ -188,6 +194,12 @@ namespace std {
     vec_pfacet::
     insert(vec_pfacet::iterator, vec_pfacet::size_type, 
 	   const vec_pfacet::value_type&);
+  template 
+    void 
+    vec_pfacet::
+    _M_fill_insert(vec_pfacet::iterator, vec_pfacet::size_type, 
+		   const vec_pfacet::value_type&);
+
 
   typedef istreambuf_iterator<char, char_traits<char> > istreambuf_iter;
   typedef ostreambuf_iterator<char, char_traits<char> > ostreambuf_iter;
@@ -213,6 +225,10 @@ namespace std {
   //
   // locale
   //
+  template 
+    bool
+    locale::operator()(const string&, const string&) const;
+
   template
     ostreambuf_iter
     _S_fill<char, ostreambuf_iter, output_iterator_tag>
@@ -231,10 +247,21 @@ namespace std {
 
   template 
     ostreambuf_iter
-    _S_format_long<char, ostreambuf_iter>
+    _S_format<char, ostreambuf_iter, unsigned long>
     (ostreambuf_iter, ios_base &, char, bool, unsigned long);
 
+#ifdef _GLIBCPP_USE_LONG_LONG
+  template
+    ostreambuf_iter
+    _S_format<char, ostreambuf_iter, unsigned long long>
+    (ostreambuf_iter, ios_base &, char, bool, unsigned long long);
+#endif
+
 #ifdef _GLIBCPP_USE_WCHAR_T
+  template 
+    bool
+    locale::operator()(const wstring&, const wstring&) const;
+
   typedef ostreambuf_iterator<wchar_t> wostreambuf_iter;
 
   template
@@ -255,25 +282,26 @@ namespace std {
 
   template 
     wostreambuf_iter
-    _S_format_long<wchar_t, wostreambuf_iter>
+    _S_format<wchar_t, wostreambuf_iter, unsigned long>
     (wostreambuf_iter, ios_base &, wchar_t, bool, unsigned long);
-#endif
 
-#if 0
-  template 
-    locale::facet ** 
-    fill_n<locale::facet **, unsigned int, locale::facet *>
-    (locale::facet **, unsigned int, locale::facet * const &);
+#ifdef _GLIBCPP_USE_LONG_LONG
+  template
+    wostreambuf_iter
+    _S_format<wchar_t, wostreambuf_iter, unsigned long long>
+    (wostreambuf_iter, ios_base &, wchar_t, bool, unsigned long long);
+#endif
+#endif // _GLIBCPP_USE_WCHAR_T
 
   template 
-    void fill<locale::facet **, locale::facet *>
-    (locale::facet **, locale::facet *, locale::facet * const &);
-#endif
+    locale::facet** 
+    fill_n<locale::facet**, unsigned int, locale::facet*>
+    (locale::facet**, unsigned int, locale::facet* const &);
+
+  template 
+    locale::facet** 
+    fill_n<locale::facet**, unsigned long, locale::facet*>
+    (locale::facet**, unsigned long, locale::facet* const &);
 
 } //std
 
-
-
-
-
-
